	LIST P=16C71
;software to sample data at RA1 at 10kHz, and output it serially at 115.2kbaud to a PC

	INCLUDE "picreg.equ"

;port A is used as follows: bit 0 - not used (tie to ground
;                           bit 1 - analogue input
;                           bit 2 - not used
;                           bit 3 - Vref
;port B is used:            bit 7 - RS232 output (TTL Low = RS232 +ve using interface)
;                           bits 6 - 0 not used (set to inputs)

;equates here

LAST_RDG    equ     0x0c
TEMP    equ     0x0d

;code starts here

	org     PIC71
boot    nop
	clrw
	movwf   INTCON          ;disable interrupts
	goto    startup         ;jump to main code
ints    retfie                  ;interrupt service routine code

startup bsf     STATUS,RP0      ;ram page 1
	movlw   b'11111111'     ;
	movwf   TRISA           ;set port a as inputs
	movlw   b'01111111'     ;set Port B:7 as output, PB6-0 input
	movwf   Port_B
	bcf     STATUS,RP0      ;ram page 0
	movlw   b'10000000'     ;set RS232 to quiescent state
	movwf   Port_B

;setup the A-D here
analogue
	movlw   b'01001001'     ;Tad = fosc/8, ch1, A/D on
	movwf   ADCON0
	bsf     STATUS,RP0
	movlw   b'00000010'     ;set RA0,1 as analogue, RA2,3 as digital
	movwf   ADCON1
	bcf     STATUS,RP0
	clrf    LAST_RDG
loop                            ;loop should be exactly 100 cycles long
	bsf     ADCON0,GO       ;set conversion going whilst we send last sample
	movlw   b'00000000'     ;1 start bit
	movwf   Port_B          ;bit time is 8.67us, so we will necessarily have +/- 0.5us jitter
	nop                     ;this bit 9us
	nop
	nop
	nop
	clrf    TEMP            ;clear output reg (not absolutely necessary)
	rrf     LAST_RDG,1      ;rotate bottom bit to carry
	rrf     TEMP,1          ;rotate carry into top bit
	movf    TEMP,0          ;get ready to output
	movwf   Port_B          ;and output bit
	nop                     ;this bit 9us (bit 0)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 8us (bit 1)
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 2)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 3)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 8us (bit 4)
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 5)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 9us (bit 6)
	nop
	nop
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movwf   Port_B
	nop                     ;this bit 8us (bit 7)
	nop
	clrf    TEMP
	rrf     LAST_RDG,1
	rrf     TEMP,1
	movf    TEMP,0
	movlw   b'10000000'
	movwf   Port_B          ;this bit at least 13us  Stop bit
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movf    ADRES,0         ;we had to wait at least 80us before getting result
	movwf   LAST_RDG
	btfsc   Port_B,3
	goto Fast
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
Fast                            ;jump to here for 10us sampling
	nop
	nop
	nop
	nop
	goto    loop            ;do this forever  (99 instructions (100 cycles))
	END

